@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 ###
-### $Release: 0.6.0 $
+### $Release: 0.7.0 $
 ### $Copyright: copyright(c) 2010-2011 kuwata-lab.com all rights reserved $
 ### $License: MIT License $
 ###
@@ -19,16 +19,16 @@ import os, re
 from glob import glob
 from kook.utils import read_file, write_file
 
-release   = prop('release', '0.6.0')
+release   = prop('release', '0.7.0')
 package   = prop('package', 'Oktest')
 copyright = prop('copyright', "copyright(c) 2010-2011 kuwata-lab.com all rights reserved")
 license   = "MIT License"
-#kook_default_product = 'test'
+kook_default_product = 'test'
 
 python = prop('python', 'python')
 
 @recipe
-@spices("-a: do with python 2.4, 2.5, 2.6, 2.7, 3.0, 3.1, 3.2a2")
+@spices("-a: do with python 2.4, 2.5, 2.6, 2.7, 3.0, 3.1, 3.2rc1")
 def task_test(c, *args, **kwargs):
     if kwargs.get('a', None):
         _do_for_all_version(c, "PYTHON=$(bin) $(bin) test/oktest_test.py")
@@ -40,10 +40,11 @@ def _do_for_all_version(c, command):
         ('2.4', '/opt/local/bin/python2.4'),
         ('2.5', '/opt/local/bin/python2.5'),
         ('2.6', '/opt/local/bin/python2.6'),
-        ('2.7', '/opt/local/bin/python2.7'),
+        #('2.7', '/opt/local/bin/python2.7'),
+        ('2.7', '/usr/local/python/2.7.1/bin/python'),
         ('3.0', '/usr/local/python/3.0.1/bin/python'),
         ('3.1', '/usr/local/python/3.1/bin/python'),
-        ('3.2', '/usr/local/python/3.2a2/bin/python'),
+        ('3.2', '/usr/local/python/3.2rc1/bin/python'),
     ]
     for ver, bin in versions:
         print("#")
@@ -64,7 +65,7 @@ def _do_for_all_version(c, command):
 
 @recipe
 @ingreds('test/doc_test.py')
-@spices("-a: do with python 2.4, 2.5, 2.6, 2.7, 3.0, 3.1, 3.2a2")
+@spices("-a: do with python 2.4, 2.5, 2.6, 2.7, 3.0, 3.1, 3.2rc1")
 def task_doc_test(c, *args, **kwargs):
     """invoke 'test/doc_test.py'"""
     if kwargs.get('a', None):
@@ -95,7 +96,7 @@ def file_doc_test_py(c):
 
 @recipe
 def task_edit(c):
-    """update 0.6.0, $Copyrigh$, and MIT License in files"""
+    """update 0.7.0, $Copyrigh$, and MIT License in files"""
     def replacer(s):
         s = re.sub(r'\$Release:[^%]*?\$',    '$Release: %s $'   % release,   s)
         s = re.sub(r'\$Copyright:[^%]*?\$',  '$Copyright: %s $' % copyright, s)
@@ -212,6 +213,7 @@ def file_CHANGES_txt(c):
 @product('Oktest.zip')
 @ingreds('website/index.html', 'website/CHANGES.txt')
 def file_Oktest_zip(c):
+    """create Oktest.zip"""
     with chdir("website"):
         system("zip -r ../Oktest.zip index.html style.css CHANGES.txt")
 
