@@ -1,5 +1,5 @@
 ###
-### $Release: 0.6.0 $
+### $Release: 0.7.0 $
 ### $Copyright: copyright(c) 2010-2011 kuwata-lab.com all rights reserved $
 ### $License: MIT License $
 ###
@@ -522,6 +522,32 @@ expected = r"""
 """[1:]
 do_test_with(desc, script, expected)
 
+###
+desc = "op 'raises' (skip AssertionError)"
+script = r"""
+from oktest import *
+class FooTest(object):
+    def test_skip_assert(self):
+        def f():
+            assert 1 == 2
+            raise ValueError('errmsg1')
+        ok (f).raises(ValueError)
+try:
+    run(FooTest)
+except AssertionError:
+    import sys
+    ex = sys.exc_info()[1]
+    print("")
+    print("ex.__class__.__name__=%r" % ex.__class__.__name__)
+    print("str(ex)=%r" % str(ex))
+"""[1:]
+expected = r"""
+* FooTest.test_skip_assert ... 
+ex.__class__.__name__='AssertionError'
+str(ex)=''
+"""[1:]
+do_test_with(desc, script, expected)
+
 ###
 desc = "op 'not raise'"
 script = r"""
@@ -555,6 +581,31 @@ expected = r"""
 """[1:]
 do_test_with(desc, script, expected)
 
+###
+desc = "op 'not raise' (skip AssertionError)"
+script = r"""
+from oktest import *
+class FooTest(object):
+    def test_skip_assert(self):
+        def f(): assert 1 == 0
+        ok (f).not_raise(Exception)              # Exception
+try:
+    run(FooTest)
+except AssertionError:
+    import sys
+    ex = sys.exc_info()[1]
+    print("")
+    print("ex.__class__.__name__=%r" % ex.__class__.__name__)
+    print("str(ex)=%r" % str(ex))
+"""[1:]
+expected = r"""
+* FooTest.test_skip_assert ... 
+ex.__class__.__name__='AssertionError'
+str(ex)=''
+"""[1:]
+do_test_with(desc, script, expected)
+
+
 ###
 desc = "op 'is_file', 'is_dir'"
 script = r"""
@@ -624,6 +675,98 @@ expected = r"""
 do_test_with(desc, script, expected)
 
 
+###
+desc = "assertion()"
+script = r"""
+import oktest
+from oktest import *
+
+@oktest.assertion
+def startswith(self, arg):
+  boolean = self.target.startswith(arg)
+  if boolean != self.expected:
+    self.failed("%r.startswith(%r) : failed." % (self.target, arg))
+
+class FooTest(object):
+  def test_original_assertion1(self):
+    ok ("foobar").startswith("foob")
+    not_ok ("foobar").startswith("a")
+  def test_original_assertion2(self):
+    ok ("foobar").startswith("afoo")
+  def test_original_assertion3(self):
+    not_ok ("foobar").startswith("foo")
+
+run(FooTest)
+"""[1:]
+expected = r"""
+* FooTest.test_original_assertion1 ... [ok]
+* FooTest.test_original_assertion2 ... [NG] 'foobar'.startswith('afoo') : failed.
+   _test_.py:15: ok ("foobar").startswith("afoo")
+* FooTest.test_original_assertion3 ... [NG] not 'foobar'.startswith('foo') : failed.
+   _test_.py:17: not_ok ("foobar").startswith("foo")
+"""[1:]
+do_test_with(desc, script, expected)
+
+
+###
+desc = "Should"
+script = r"""
+from oktest import *
+
+class FooTest(object):
+  def test_should1(self):
+    ok ("foobar").should.startswith("foob")
+  def test_should2(self):
+    ok ("foobar").should.startswith("aaa")
+  def test_should3(self):
+    ok ("foobar").should.start_with("foob")   # AttributeError
+  def test_should4(self):
+    ok (Bar('sasaki')).should.name()          # ValueError: not a callable
+  def test_should5(self):
+    ok ("Sasaki").should.upper()              # ValueError: expected to return True or False
+
+class Bar(object):
+  def __init__(self, name):
+    self.name = name
+
+run(FooTest)
+"""[1:]
+expected = r"""
+* FooTest.test_should1 ... [ok]
+* FooTest.test_should2 ... [NG] 'foobar'.startswith('aaa') : failed.
+   /Users/kwatch/src/oktest/python/lib/oktest.py:505: func(obj)
+* FooTest.test_should3 ... [ERROR] AttributeError: 'str' object has no attribute 'start_with'
+  - _test_.py:9:  ok ("foobar").should.start_with("foob")   # AttributeError
+  - /Users/kwatch/src/oktest/python/lib/oktest.py:419:  val = getattr(ass.target, key)
+* FooTest.test_should4 ... [ERROR] ValueError: Bar.name: not a callable.
+  - _test_.py:11:  ok (Bar('sasaki')).should.name()          # ValueError: not a callable
+  - /Users/kwatch/src/oktest/python/lib/oktest.py:422:  raise ValueError(msg)
+* FooTest.test_should5 ... [ERROR] ValueError: 'Sasaki'.upper(): expected to return True or False but it returned 'SASAKI'.
+  - _test_.py:13:  ok ("Sasaki").should.upper()              # ValueError: expected to return True or False
+  - /Users/kwatch/src/oktest/python/lib/oktest.py:430:  raise ValueError(msg)
+"""[1:]
+do_test_with(desc, script, expected, re.compile(r'.*oktest\.py:\d+\:.*\n'))
+
+###
+desc = "Should_not"
+script = r"""
+from oktest import *
+
+class FooTest(object):
+  def test_should_not1(self):
+    ok ("foobar").should_not.startswith("aaa")
+  def test_should_not2(self):
+    ok ("foobar").should_not.startswith("foo")
+
+run(FooTest)
+"""[1:]
+expected = r"""
+* FooTest.test_should_not1 ... [ok]
+* FooTest.test_should_not2 ... [NG] not 'foobar'.startswith('foo') : failed.
+   /Users/kwatch/src/oktest/python/lib/oktest.py:505: func(obj)
+"""[1:]
+do_test_with(desc, script, expected, re.compile(r'.*oktest\.py:\d+\:.*\n'))
+
 
 ### run (with class objects)
 desc = "run (with class objects)"
@@ -686,6 +829,30 @@ expected = r"""
 """[1:]
 do_test_with(desc, script, expected)
 
+### run (skip AssertionError)
+desc = "run (skip AssertionError)"
+script = r"""
+from oktest import *
+class FooTestCase(object):
+    def test_1(self):
+        assert 1 != 1
+        ok (1+1) == 2
+try:
+    run()
+except Exception:
+    import sys
+    ex = sys.exc_info()[1]
+    print("")
+    print("ex.__class__.__name__=%r" % ex.__class__.__name__)
+    print("str(ex)=%r" % str(ex))
+"""
+expected = r"""
+* FooTestCase.test_1 ... 
+ex.__class__.__name__='AssertionError'
+str(ex)=''
+"""[1:]
+do_test_with(desc, script, expected)
+
 ## _min_firstlineno_of_methods
 desc = "_min_firstlineno_of_methods"
 script = r"""
@@ -1019,7 +1186,7 @@ class FooTest(object):
                 ok (os.path.basename(os.getcwd())) == '_dummy_.d'
                 ok (os.getcwd()) != pwd
                 ok (os.getcwd()) == os.path.join(pwd, '_dummy_.d')
-            chdir('_dummy_.d')(g)
+            chdir('_dummy_.d', g)
             ok (os.getcwd()) == pwd
         dummy_dir('_dummy_.d')(f)
 run(FooTest)
@@ -1253,6 +1420,33 @@ True
 """[1:]
 do_test_with(desc, script, expected)
 
+### Tracer (repr style)
+desc = "Tracer (repr style)"
+script = r"""
+from oktest import *
+from oktest.tracer import Tracer
+class Dummy(object):
+    def f1(self, *args, **kwargs):
+        return 1
+    def __repr__(self):
+        return '#<Dummy>'
+tr = Tracer()
+obj = Dummy()
+tr.trace_method(obj, 'f1')
+obj.f1(10,20,x=30)
+print(repr(tr[0]))
+tr[0] == []
+print(repr(tr[0]))
+tr[0] == ()
+print(repr(tr[0]))
+"""[1:]
+expected = """
+f1(10, 20, x=30) #=> 1
+[#<Dummy>, 'f1', (10, 20), {'x': 30}, 1]
+(#<Dummy>, 'f1', (10, 20), {'x': 30}, 1)
+"""[1:]
+do_test_with(desc, script, expected)
+
 ### Tracer (fake)
 desc = "Tracer (fake_func)"
 script = r"""
